#!/usr/bin/env python
# -*- coding: utf-8 -*-
# ===============================
# Author  : Konsta Vesterinen

import re
import inspect
import itertools
from collections import OrderedDict

from decorator import decorator


class ValidationFailure(Exception):
    def __init__(self, func, args):
        self.func = func
        self.__dict__.update(args)

    def __repr__(self):
        return "ValidationFailure(func={func}, args={args})".format(
            func=self.func.__name__, args=dict([(k, v) for (k, v) in self.__dict__.items() if k != "func"])
        )

    def __str__(self):
        return repr(self)

    def __unicode__(self):
        return repr(self)

    def __bool__(self):
        return False

    def __nonzero__(self):
        return False


def func_args_as_dict(func, args, kwargs):
    """
    Return given function's positional and key value arguments as an ordered
    dictionary.
    """
    _getargspec = inspect.getfullargspec

    arg_names = list(OrderedDict.fromkeys(itertools.chain(_getargspec(func)[0], kwargs.keys())))
    return OrderedDict(list(zip(arg_names, args)) + list(kwargs.items()))


def validator(func, *args, **kwargs):
    """
    A decorator that makes given function validator.
    Whenever the given function is called and returns ``False`` value
    this decorator returns :class:`ValidationFailure` object.
    Example::
        >>> @validator
        ... def even(value):
        ...     return not (value % 2)
        >>> even(4)
        True
        >>> even(5)
        ValidationFailure(func=even, args={'value': 5})
    :param func: function to decorate
    :param args: positional function arguments
    :param kwargs: key value function arguments
    """

    def wrapper(func, *args, **kwargs):
        value = func(*args, **kwargs)
        if not value:
            return ValidationFailure(func, func_args_as_dict(func, args, kwargs))
        return True

    return decorator(wrapper, func)


ip_middle_octet = r"(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5]))"
ip_last_octet = r"(?:\.(?:0|[1-9]\d?|1\d\d|2[0-4]\d|25[0-5]))"

regex = re.compile(  # noqa: W605
    r"^"
    # protocol identifier
    r"(?:(?:https?|ftp)://)"
    # user:pass authentication
    r"(?:[-a-z\u00a1-\uffff0-9._~%!$&'()*+,;=:]+" r"(?::[-a-z0-9._~%!$&'()*+,;=:]*)?@)?" r"(?:" r"(?P<private_ip>"
    # IP address exclusion
    # private & local networks
    r"(?:(?:10|127)" + ip_middle_octet + r"{2}" + ip_last_octet + r")|"
    r"(?:(?:169\.254|192\.168)" + ip_middle_octet + ip_last_octet + r")|"
    r"(?:172\.(?:1[6-9]|2\d|3[0-1])" + ip_middle_octet + ip_last_octet + r"))"
    r"|"
    # private & local hosts
    r"(?P<private_host>" r"(?:localhost))" r"|"
    # IP address dotted notation octets
    # excludes loopback network 0.0.0.0
    # excludes reserved space >= 224.0.0.0
    # excludes network & broadcast addresses
    # (first & last IP address of each class)
    r"(?P<public_ip>" r"(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])" r"" + ip_middle_octet + r"{2}" r"" + ip_last_octet + r")" r"|"
    # IPv6 RegEx from https://stackoverflow.com/a/17871737
    r"\[("
    # 1:2:3:4:5:6:7:8
    r"([0-9a-fA-F]{1,4}:){7,7}[0-9a-fA-F]{1,4}|"
    # 1::                              1:2:3:4:5:6:7::
    r"([0-9a-fA-F]{1,4}:){1,7}:|"
    # 1::8             1:2:3:4:5:6::8  1:2:3:4:5:6::8
    r"([0-9a-fA-F]{1,4}:){1,6}:[0-9a-fA-F]{1,4}|"
    # 1::7:8           1:2:3:4:5::7:8  1:2:3:4:5::8
    r"([0-9a-fA-F]{1,4}:){1,5}(:[0-9a-fA-F]{1,4}){1,2}|"
    # 1::6:7:8         1:2:3:4::6:7:8  1:2:3:4::8
    r"([0-9a-fA-F]{1,4}:){1,4}(:[0-9a-fA-F]{1,4}){1,3}|"
    # 1::5:6:7:8       1:2:3::5:6:7:8  1:2:3::8
    r"([0-9a-fA-F]{1,4}:){1,3}(:[0-9a-fA-F]{1,4}){1,4}|"
    # 1::4:5:6:7:8     1:2::4:5:6:7:8  1:2::8
    r"([0-9a-fA-F]{1,4}:){1,2}(:[0-9a-fA-F]{1,4}){1,5}|"
    # 1::3:4:5:6:7:8   1::3:4:5:6:7:8  1::8
    r"[0-9a-fA-F]{1,4}:((:[0-9a-fA-F]{1,4}){1,6})|"
    # ::2:3:4:5:6:7:8  ::2:3:4:5:6:7:8 ::8       ::
    r":((:[0-9a-fA-F]{1,4}){1,7}|:)|"
    # fe80::7:8%eth0   fe80::7:8%1
    # (link-local IPv6 addresses with zone index)
    r"fe80:(:[0-9a-fA-F]{0,4}){0,4}%[0-9a-zA-Z]{1,}|"
    r"::(ffff(:0{1,4}){0,1}:){0,1}"
    r"((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}"
    # ::255.255.255.255   ::ffff:255.255.255.255  ::ffff:0:255.255.255.255
    # (IPv4-mapped IPv6 addresses and IPv4-translated addresses)
    r"(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])|"
    r"([0-9a-fA-F]{1,4}:){1,4}:"
    r"((25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])\.){3,3}"
    # 2001:db8:3:4::192.0.2.33  64:ff9b::192.0.2.33
    # (IPv4-Embedded IPv6 Address)
    r"(25[0-5]|(2[0-4]|1{0,1}[0-9]){0,1}[0-9])" r")\]|"
    # host name
    r"(?:(?:(?:xn--[-]{0,2})|[a-z\u00a1-\uffff\U00010000-\U0010ffff0-9]-?)*" r"[a-z\u00a1-\uffff\U00010000-\U0010ffff0-9]+)"
    # domain name
    r"(?:\.(?:(?:xn--[-]{0,2})|[a-z\u00a1-\uffff\U00010000-\U0010ffff0-9]-?)*"
    r"[a-z\u00a1-\uffff\U00010000-\U0010ffff0-9]+)*"
    # TLD identifier
    r"(?:\.(?:(?:xn--[-]{0,2}[a-z\u00a1-\uffff\U00010000-\U0010ffff0-9]{2,})|"
    r"[a-z\u00a1-\uffff\U00010000-\U0010ffff]{2,}))"
    r")"
    # port number
    r"(?::\d{2,5})?"
    # resource path
    r"(?:/[-a-z\u00a1-\uffff\U00010000-\U0010ffff0-9._~%!$&'()*+,;=:@/]*)?"
    # query string
    r"(?:\?\S*)?"
    # fragment
    r"(?:#\S*)?" r"$",
    re.UNICODE | re.IGNORECASE,
)

pattern = re.compile(regex)


@validator
def url_validator(value, public=False):
    """
    Return whether or not given value is a valid URL.
    If the value is valid URL this function returns ``True``, otherwise
    :class:`~validators.utils.ValidationFailure`.
    This validator is based on the wonderful `URL validator of dperini`_.
    .. _URL validator of dperini:
        https://gist.github.com/dperini/729294
    Examples::
        >>> url('http://foobar.dk')
        True
        >>> url('ftp://foobar.dk')
        True
        >>> url('http://10.0.0.1')
        True
        >>> url('http://foobar.d')
        ValidationFailure(func=url, ...)
        >>> url('http://10.0.0.1', public=True)
        ValidationFailure(func=url, ...)
    .. versionadded:: 0.2
    .. versionchanged:: 0.10.2
        Added support for various exotic URLs and fixed various false
        positives.
    .. versionchanged:: 0.10.3
        Added ``public`` parameter.
    .. versionchanged:: 0.11.0
        Made the regular expression this function uses case insensitive.
    .. versionchanged:: 0.11.3
        Added support for URLs containing localhost
    :param value: URL address string to validate
    :param public: (default=False) Set True to only allow a public IP address
    """
    result = pattern.match(value)
    if not public:
        return result

    return result and not any((result.groupdict().get(key) for key in ("private_ip", "private_host")))



import requests
from ..exceptions import InvalidParams
import logging

logger = logging.getLogger("__name__")


def baato_map_style(*args, **kwargs):
    baato_url = kwargs.pop("baseURL")
    access_key = kwargs.pop("access_token")
    if "style_name" not in kwargs:
        raise InvalidParams("style_name parameter must be provide for search request")

    try:
        style_name = kwargs.pop("style_name")
        res = requests.get(f"{baato_url}styles/{style_name.lower()}?key={access_key}")
        return res
    except requests.exceptions.RequestException as err:
        logger.debug(f"Exception occure in map style API {err}")



import requests
from ..exceptions import InvalidParams
import logging

logger = logging.getLogger("__name__")


def baato_search(*args, **kwargs):
    baato_url = kwargs.pop("baseURL")
    access_key = kwargs.pop("access_token")
    if "q" not in kwargs:
        raise InvalidParams("q parameter must be provide for search request")
    try:
        params = ""
        for k, v in kwargs.items():
            params += f"{k}={v}&"
        res = requests.get(f"{baato_url}search?key={access_key}&{params}")
        return res
    except requests.exceptions.RequestException as err:
        logger.debug(f"Exception occure in search API {err}")



import logging
from logging import NullHandler
from baato.main import BaatoClient

__all__ = ("BaatoClient",)

# Set default logging handler to avoid "No handler found" warnings.
logging.getLogger(__name__).addHandler(NullHandler())



"""Check the latest version at https://pypi.org/project/baato/"""
__version__ = "1.0.1"
__pypi_username__ = "baato"
__pypi_packagename__ = "baato"
__github_username__ = "baato"
__github_reponame__ = "python-client"
__baato_endpoint__ = "https://api.baato.io/api"
__baato_version__ = "v1"



from baato.resources.search import baato_search
from baato.resources.reverse_search import baato_reverse_search
from baato.resources.places import baato_places
from baato.resources.near_by_places import baato_near_by
from baato.resources.direction import baato_direction
from baato.resources.map_style import baato_map_style
from baato.resources.validators import url_validator

__all__ = (
    "baato_search",
    "baato_reverse_search",
    "baato_places",
    "baato_near_by",
    "baato_direction",
    "baato_map_style",
    "url_validator",
)



import pytest
from baato import BaatoClient
from .utils import BAATO_ACCESS_TOKEN


def test_direction():
    res = BaatoClient(access_token=BAATO_ACCESS_TOKEN).direction(
        points=["27.71772,85.32784", "27.73449,85.33714"], mode="car"
    )
    assert res["status"] == 200


def test_direction_without_query():
    with pytest.raises(Exception):
        BaatoClient(access_token=BAATO_ACCESS_TOKEN).direction()


def test_direction_only_points():
    with pytest.raises(Exception):
        BaatoClient(access_token=BAATO_ACCESS_TOKEN).direction(points=["27.71772,85.32784", "27.73449,85.33714"])


def test_direction_only_mode():
    with pytest.raises(Exception):
        BaatoClient(access_token=BAATO_ACCESS_TOKEN).direction(mode="car")



import requests
from ..exceptions import InvalidParams
import logging

logger = logging.getLogger("__name__")


def baato_places(*args, **kwargs):
    baato_url = kwargs.pop("baseURL")
    access_key = kwargs.pop("access_token")
    if "place_id" not in kwargs:
        raise InvalidParams("place_id parameter must be provide for search request")

    try:
        placeId = kwargs.pop("place_id")
        res = requests.get(f"{baato_url}places?key={access_key}&placeId={placeId}")
        return res
    except requests.exceptions.RequestException as err:
        logger.debug(f"Exception occure in places API {err}")



import pytest
from baato import BaatoClient
from .utils import BAATO_ACCESS_TOKEN


def test_map_style():
    res = BaatoClient(access_token=BAATO_ACCESS_TOKEN).map_style(style_name="monochrome")
    assert type(res) is dict


def test_map_style_without_query():
    with pytest.raises(Exception):
        BaatoClient(access_token=BAATO_ACCESS_TOKEN).map_style()



import pytest
import requests
from baato.config import __baato_endpoint__
from baato import BaatoClient
from .utils import BAATO_ACCESS_TOKEN


def test_endpoint():
    res = requests.get(__baato_endpoint__)
    assert res.status_code == 401


def test_client_without_token():
    with pytest.raises(Exception):
        BaatoClient()


def test_client_endpoint():
    client = BaatoClient(access_token=BAATO_ACCESS_TOKEN)
    assert client.endpoint == "https://api.baato.io/api"


def test_client_version():
    client = BaatoClient(access_token=BAATO_ACCESS_TOKEN)
    assert client.version == "v1"


def test_client_custom_url():
    res = BaatoClient(access_token=BAATO_ACCESS_TOKEN, endpoint="http://baato.io")
    assert res.endpoint == "http://baato.io"


def test_client_custom_url_exception():
    with pytest.raises(Exception):
        BaatoClient(access_token=BAATO_ACCESS_TOKEN, endpoint="test")


def test_client_token_invalid_token():
    res = BaatoClient(access_token="TEST").search(q="kathmandu")
    assert res["status"] == 403



import pytest
from baato import BaatoClient
from .utils import BAATO_ACCESS_TOKEN


def test_places():
    res = BaatoClient(access_token=BAATO_ACCESS_TOKEN).places(place_id=100006)
    assert res["status"] == 200


def test_places_without_query():
    with pytest.raises(Exception):
        BaatoClient(access_token=BAATO_ACCESS_TOKEN).places()



import logging
from baato import exceptions
from baato.resources import baato_search
from baato.resources import baato_reverse_search
from baato.resources import baato_places
from baato.resources import baato_near_by
from baato.resources import baato_direction
from baato.resources import baato_map_style
from baato.resources import url_validator
from baato.config import __baato_endpoint__, __baato_version__

logger = logging.getLogger(__name__)


class BaatoApi(object):
    def __init__(self, access_token=None, endpoint=None, version=None, *args, **kwargs):
        """
        Create API object
        """
        self.endpoint = self.validate_endpoint(endpoint) if endpoint else __baato_endpoint__
        self.version = self.validate_version(version) if version else __baato_version__
        self.access_token = self.validate_access_token(access_token)

        self.baseURL = self.get_baseURL(self.endpoint, self.version)

    @staticmethod
    def validate_access_token(access_token):
        if not access_token:
            raise exceptions.InvalidConfig("access_token must provide")
        return access_token

    @staticmethod
    def validate_endpoint(endpoint):
        if endpoint and url_validator(endpoint):
            return endpoint.rstrip("/")
        else:
            raise exceptions.InvalidConfig("Endpoint is Invalid", "Received: %s" % (endpoint))

    @staticmethod
    def validate_version(version):
        if version and version in [
            "v1",
        ]:
            return version
        raise exceptions.InvalidConfig("Version is Invalid", "Received: %s" % (version))

    @staticmethod
    def get_baseURL(endpoint, version):
        """Create the absolute baato API url

        Args:
            endpoint (str):
            version (str):

        Returns:
            str:
        """
        return "%s/%s/" % (endpoint, version)

    def search(self, *args, **params):
        """Baato search API # https://docs.baato.io/#/v1/services/search
        Returns:
            json:
        """
        response = baato_search(access_token=self.access_token, baseURL=self.baseURL, **params)
        return response.json()

    def reverse(self, *args, **params):
        """Baato Reverse API # https://docs.baato.io/#/v1/services/reverse

        Returns:
            json:
        """
        response = baato_reverse_search(access_token=self.access_token, baseURL=self.baseURL, **params)
        return response.json()

    def places(self, *args, **params):
        """Baato Places API # https://docs.baato.io/#/v1/services/places

        Returns:
            json:
        """
        response = baato_places(access_token=self.access_token, baseURL=self.baseURL, **params)
        return response.json()

    def near_by(self, *args, **params):
        """Baato NearBy API # https://docs.baato.io/#/v1/services/nearby_places

        Returns:
            json:
        """
        response = baato_near_by(access_token=self.access_token, baseURL=self.baseURL, **params)
        return response.json()

    def direction(self, *args, **params):
        """Baato Direction API # https://docs.baato.io/#/v1/services/directions

        Returns:
            json:
        """
        response = baato_direction(access_token=self.access_token, baseURL=self.baseURL, **params)
        return response.json()

    def map_style(self, *args, **params):
        """Baato Map style API # https://docs.baato.io/#/v1/services/styles

        Returns:
            json:
        """
        response = baato_map_style(access_token=self.access_token, baseURL=self.baseURL, **params)
        return response.json()


class BaatoClient(BaatoApi):
    pass



import requests
from ..exceptions import InvalidParams
import logging

logger = logging.getLogger("__name__")


def baato_direction(*args, **kwargs):
    baato_url = kwargs.pop("baseURL")
    access_key = kwargs.pop("access_token")
    if any(qparm not in kwargs for qparm in ("points", "mode")):
        raise InvalidParams("points and mode parameters must be provide for search request")

    try:
        points = kwargs.pop("points")
        params = ""
        for param_name, value in kwargs.items():
            params += f"{param_name}={value}&"
        for point in points:
            params += f"points[]={point}&"
        res = requests.get(f"{baato_url}directions?key={access_key}&{params}")
        return res
    except requests.exceptions.RequestException as err:
        logger.debug(f"Exception occure in direction API {err}")



import pytest
from baato import BaatoClient
from .utils import BAATO_ACCESS_TOKEN


def test_near_by():
    res = BaatoClient(access_token=BAATO_ACCESS_TOKEN).near_by(lat=27.70446921370009, lon=85.32051086425783, type="school")
    assert res["status"] == 200


def test_reverse_without_query():
    with pytest.raises(Exception):
        BaatoClient(access_token=BAATO_ACCESS_TOKEN).near_by()


def test_near_by_only_lat():
    with pytest.raises(Exception):
        BaatoClient(access_token=BAATO_ACCESS_TOKEN).near_by(lat=27.70446921370009)


def test_near_by_only_lon():
    with pytest.raises(Exception):
        BaatoClient(access_token=BAATO_ACCESS_TOKEN).near_by(lon=85.32051086425783)


def test_near_by_only_type():
    with pytest.raises(Exception):
        BaatoClient(access_token=BAATO_ACCESS_TOKEN).near_by(type="school")



# -*- coding: utf-8 -*-
import codecs
import os
import subprocess
import sys
from shutil import rmtree

from setuptools import setup, find_packages, Command
from baato import config

here = os.path.abspath(os.path.dirname(__file__))

long_description = ""
with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as readme:
    long_description = readme.read()


class BaseCommand(Command):
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print("\033[1m{0}\033[0m".format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def _run(self, s, command):
        try:
            self.status(s + "\n" + " ".join(command))
            subprocess.check_call(command)
        except subprocess.CalledProcessError as error:
            sys.exit(error.returncode)


class ValidateCommand(BaseCommand):
    """Support setup.py validate."""

    description = "Run Python static code analyzer (flake8), formatter (black) and unit tests (pytest)."

    user_options = [("test-target=", "i", "tests/{test-target}")]

    def initialize_options(self):
        self.test_target = ""

    def run(self):
        self._run(
            "Installing test dependencies ...",
            [sys.executable, "-m", "pip", "install", "-r", f"{here}/requirements.txt"],
        )
        self._run("Running black ...", [sys.executable, "-m", "black", f"{here}/baato"])
        self._run("Running flake8 for legacy packages ...", [sys.executable, "-m", "flake8", f"{here}/baato"])

        self._run(
            "Running unit tests ...",
            [
                sys.executable,
                "-m",
                "pytest",
            ],
        )


class UploadCommand(BaseCommand):
    """Support setup.py upload."""

    description = "Build and publish the package."

    def run(self):
        self._run(
            "Installing upload dependencies ...",
            [sys.executable, "-m", "pip", "install", "wheel"],
        )
        try:
            self.status("Removing previous builds ...")
            rmtree(os.path.join(here, "dist"))
            rmtree(os.path.join(here, "build"))
        except OSError:
            pass

        self._run(
            "Building Source and Wheel (universal) distribution ...",
            [sys.executable, "setup.py", "sdist", "bdist_wheel", "--universal"],
        )
        self._run(
            "Installing Twine dependency ...",
            [sys.executable, "-m", "pip", "install", "twine"],
        )
        self._run(
            "Uploading the package to PyPI via Twine ...",
            [sys.executable, "-m", "twine", "upload", "dist/*"],
        )


setup(
    name="baato",
    version=config.__version__,
    description="Baato API for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Topic :: Scientific/Engineering :: GIS",
        "Topic :: Software Development",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
    ],
    keywords=["Baato", "Python", "DRY"],
    author="Kathmandu Living Labs Consult",
    author_email="klltech@gmail.com",
    url="https://github.com/baato/python-client",
    license="MIT License",
    packages=find_packages(
        exclude=[
            "docs",
            "tests",
            "tests.*",
            "tutorial",
        ]
    ),
    include_package_data=True,
    install_requires=[
        "requests",
        "decorator",
        "certifi",
    ],
    python_requires=">=3.6",
    setup_requires=["setuptools_scm"],
    cmdclass={
        "validate": ValidateCommand,
        "upload": UploadCommand,
    },
)



import pytest
from baato import BaatoClient
from .utils import BAATO_ACCESS_TOKEN


def test_search():
    res = BaatoClient(access_token=BAATO_ACCESS_TOKEN).search(q="kathmandu")
    assert res["status"] == 200


def test_search_without_query():
    with pytest.raises(Exception):
        BaatoClient(access_token=BAATO_ACCESS_TOKEN).search()
